@@ -506,110 +506,6 @@ usefulness = function(pop,trait=1,use="gv",p=0.1,
   return(mean(response))
 }
 
-#' @title Writes a Pop-class as PLINK files
-#' 
-#' @description
-#' Writes a Pop-class as PLINK PED and MAP files
-#'
-#' @param pop an object of \code{\link{Pop-class}}
-#' @param baseName a character. Basename of PED and MAP files.
-#' @param trait an integer. Which phenotype trait should be used.
-#' @param snpChip an integer. Which SNP array should be used.
-#' @param simParam an object of \code{\link{SimParam}}
-#' @param chromLength an integer. The size of chromosomes in base
-#' pairs; assuming all chromosomes are of the same size.
-#'
-#' @examples 
-#' \dontrun{
-#' #Create founder haplotypes
-#' founderPop = quickHaplo(nInd=10, nChr=1, segSites=10)
-#' 
-#' #Set simulation parameters
-#' SP = SimParam$new(founderPop)
-#' SP$setSexes(sex = "yes_rand")
-#' SP$addTraitA(nQtlPerChr = 10)
-#' SP$addSnpChip(nSnpPerChr = 5)
-#' 
-#' #Create population
-#' pop = newPop(rawPop = founderPop)
-#' pop = setPheno(pop, varE = SP$varA)
-#' writePlink(pop, baseName="test")
-#' 
-#' #Test
-#' test = read.table(file = "test.ped")
-#' #...sex
-#' if (!identical(x = c("M", "F")[test[[5]]], y = pop@sex)) { stop() }
-#' #...pheno (issues with rounding)
-#' # if (!identical(x = test[[6]], y = pop@pheno[, 1])) { stop() }
-#' #...genotypes
-#' x = test[, -(1:6)]  - 1
-#' x[, 1] = x[, 1] + x[, 2]
-#' x[, 2] = x[, 3] + x[, 4]
-#' x[, 3] = x[, 5] + x[, 6]
-#' x[, 4] = x[, 7] + x[, 8]
-#' x[, 5] = x[, 9] + x[, 10]
-#' y = pullSnpGeno(pop)
-#' if (sum(x[, 1:5] - y) != 0) { stop() }
-#' }
-#' @export
-writePlink = function(pop, baseName, trait = 1L, snpChip = 1L, simParam = NULL,
-                      chromLength = 10L^8) {
-  if (is.null(simParam)) {
-    simParam = get(x = "SP", envir = .GlobalEnv)
-  }
-  if (pop@ploidy != 2L) {
-    stop(paste0("writePlink() will write ", pop@ploidy, " alleles for each locus!"))
-  }  
-  
-  # ---- Map ----
-  
-  # This assumes equal number of markers per chromosome!
-  map = data.frame(chr = rep(x = 1L:simParam$nChr,
-                             each = simParam$snpChips[[snpChip]]@lociPerChr[1L]),
-                   loc = paste0("SNP_", 1L:simParam$snpChips[[snpChip]]@nLoci),
-                   posGenetic = 0L,
-                   pos = simParam$snpChips[[snpChip]]@lociLoc)
-  for (chr in 1L:simParam$nChr) {
-    # chr = 1
-    sel = map$chr == chr
-    map$posGenetic[sel] = simParam$genMap[[chr]][map$pos[sel]]
-  }
-  map$pos = round(map$posGenetic * chromLength)
-  write.table(x = map, file = paste0(baseName, ".map"),
-              col.names = FALSE, row.names = FALSE, quote = FALSE)
-  
-  # ---- Ped ----
-  
-  # First the FAM format, which covers the first 6 columns of the PED format
-  fam = data.frame(family = rep(x = 1L, times = pop@nInd),
-                   id     = as.integer(pop@id),
-                   father = as.integer(pop@father),
-                   mother = as.integer(pop@mother),
-                   sex = 0L,
-                   pheno  = 0)
-  if (!any(pop@sex == "H")) {
-    fam$sex = (pop@sex == "F") + 1L
-  }
-  if (!anyNA(pop@pheno[, trait])) {
-    fam$pheno = pop@pheno[, trait]
-  }
-  
-  # Select loci on the SNP array
-  tmp = selectLoci(chr          = 1L:simParam$nChr,
-                   inLociPerChr = simParam$snpChips[[snpChip]]@lociPerChr,
-                   inLociLoc    = simParam$snpChips[[snpChip]]@lociLoc)
-  # Add loci alleles to fam and write to file directly from C++
-  writePlinkPed(fam    = fam,
-                haplo  = getHaplo(geno       = pop@geno,
-                                  lociPerChr = tmp$lociPerChr,
-                                  lociLoc    = tmp$lociLoc,
-                                  nThreads   = simParam$nThreads),
-                nInd   = nrow(fam),
-                ploidy = pop@ploidy,
-                nLoc   = sum(tmp$lociPerChr),
-                file   = paste0(baseName, ".ped"))
-}
-
 #' @title Linear transformation matrix
 #' 
 #' @description 
