@@ -16,7 +16,7 @@ arma::Mat<unsigned char> getGeno(const arma::field<arma::Cube<unsigned char> >&
   arma::uword nInd = geno(0).n_slices;
   arma::uword nChr = geno.n_elem;
   arma::uword ploidy = geno(0).n_cols;
-  if(nInd<nThreads){
+  if(nInd < static_cast<arma::uword>(nThreads) ){
     nThreads = nInd;
   }
   arma::Mat<unsigned char> output(nInd,arma::sum(lociPerChr),arma::fill::zeros);
@@ -53,6 +53,56 @@ arma::Mat<unsigned char> getGeno(const arma::field<arma::Cube<unsigned char> >&
   return output;
 }
 
+// Extracts genotypes for a specified subset of individuals (indVec)
+// The subset does not need to be ordered and is returned in the order submitted
+// // [[Rcpp::export]]
+// arma::Mat<unsigned char> getGenoSubset(const arma::field<arma::Cube<unsigned char> >& geno, 
+//                                        arma::uvec indVec,
+//                                        const arma::Col<int>& lociPerChr,
+//                                        arma::uvec lociLoc, int nThreads){
+//   // R to C++ index correction
+//   lociLoc -= 1;
+//   
+//   arma::uword nInd = indVec.n_elem;
+//   arma::uword nChr = geno.n_elem;
+//   arma::uword ploidy = geno(0).n_cols;
+//   if(nInd < static_cast<arma::uword>(nThreads) ){
+//     nThreads = nInd;
+//   }
+//   arma::Mat<unsigned char> output(nInd,arma::sum(lociPerChr),arma::fill::zeros);
+//   int loc1;
+//   int loc2 = -1;
+//   for(arma::uword i=0; i<nChr; ++i){
+//     if(lociPerChr(i)>0){
+//       // Get loci locations
+//       loc1 = loc2+1;
+//       loc2 += lociPerChr(i);
+//       arma::uvec chrLociLoc = lociLoc(arma::span(loc1,loc2));
+// #ifdef _OPENMP
+// #pragma omp parallel for schedule(static) num_threads(nThreads)
+// #endif
+//       for(arma::uword ind=0; ind<nInd; ++ind){
+//         std::bitset<8> workBits;
+//         arma::uword currentByte, newByte;
+//         for(arma::uword p=0; p<ploidy; ++p){
+//           currentByte = chrLociLoc(0)/8;
+//           workBits = toBits(geno(i)(currentByte,p,indVec(ind)));
+//           output(ind,loc1) += (unsigned char) workBits[chrLociLoc(0)%8];
+//           for(arma::uword j=1; j<chrLociLoc.n_elem; ++j){
+//             newByte = chrLociLoc(j)/8;
+//             if(newByte != currentByte){
+//               currentByte = newByte;
+//               workBits = toBits(geno(i)(currentByte,p,indVec(ind)));
+//             }
+//             output(ind,j+loc1) += (unsigned char) workBits[chrLociLoc(j)%8];
+//           }
+//         }
+//       }
+//     }
+//   }
+//   return output;
+// }
+
 // [[Rcpp::export]]
 arma::Mat<unsigned char> getMaternalGeno(const arma::field<arma::Cube<unsigned char> >& geno, 
                                          const arma::Col<int>& lociPerChr,
@@ -63,7 +113,7 @@ arma::Mat<unsigned char> getMaternalGeno(const arma::field<arma::Cube<unsigned c
   arma::uword nInd = geno(0).n_slices;
   arma::uword nChr = geno.n_elem;
   arma::uword ploidy = geno(0).n_cols;
-  if(nInd<nThreads){
+  if(nInd < static_cast<arma::uword>(nThreads) ){
     nThreads = nInd;
   }
   arma::Mat<unsigned char> output(nInd,arma::sum(lociPerChr),arma::fill::zeros);
@@ -110,7 +160,7 @@ arma::Mat<unsigned char> getPaternalGeno(const arma::field<arma::Cube<unsigned c
   arma::uword nInd = geno(0).n_slices;
   arma::uword nChr = geno.n_elem;
   arma::uword ploidy = geno(0).n_cols;
-  if(nInd<nThreads){
+  if(nInd < static_cast<arma::uword>(nThreads) ){
     nThreads = nInd;
   }
   arma::Mat<unsigned char> output(nInd,arma::sum(lociPerChr),arma::fill::zeros);
@@ -158,7 +208,7 @@ arma::Mat<unsigned char> getHaplo(const arma::field<arma::Cube<unsigned char> >&
   arma::uword nInd = geno(0).n_slices;
   arma::uword nChr = geno.n_elem;
   arma::uword ploidy = geno(0).n_cols;
-  if(nInd<nThreads){
+  if(nInd < static_cast<arma::uword>(nThreads) ){
     nThreads = nInd;
   }
   arma::Mat<unsigned char> output(nInd*ploidy,arma::sum(lociPerChr));
@@ -207,7 +257,7 @@ arma::Mat<unsigned char> getOneHaplo(const arma::field<arma::Cube<unsigned char>
   
   arma::uword nInd = geno(0).n_slices;
   arma::uword nChr = geno.n_elem;
-  if(nInd<nThreads){
+  if(nInd < static_cast<arma::uword>(nThreads) ){
     nThreads = nInd;
   }
   arma::Mat<unsigned char> output(nInd,arma::sum(lociPerChr));
